/**
 * @file    mg32l003_vc.h
 * @author  megawin Application Team
 * @version V0.0.2
 * @date    16-March-2023
 * @brief   This file contains all the functions prototypes for the VC firmware
 *          library.
 */

/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef __MG32L003_VC_H
#define __MG32L003_VC_H

#ifdef __cplusplus
 extern "C" {
#endif

/* Includes ------------------------------------------------------------------*/
#include "mg32l003.h"

/** @addtogroup VC
  * @{
  */

/* Exported types ------------------------------------------------------------*/

/**
  * @brief  VC Init structure definition
  */
typedef struct
{
  uint32_t V25div;           /*!< Specifies the VC V25DIV divider, @ref VC_VC25_Division_Slection */
  uint32_t NegPinSel;        /*!< Negative input select, @ref VC_InPut_Selection*/
  uint32_t PosPinSel;        /*!< Positive input select, @ref VC_InPut_Selection */
  uint32_t Filter_Enable;    /*!< Filter counter Enable or Disable, @ref VC_Filter_Function_Selection*/
  uint32_t Filter_Clk;       /*!< Filter clk select, @ref VC_Filter_Clock_Selection*/
  uint32_t Fliter_Num;       /*!< Filter counter number. Min_Data >= 0x00 and Max_Data = 0xFFFF*/
} VC_InitTypeDef;

/* Exported constants --------------------------------------------------------*/

/** @defgroup VC_Exported_Constants
  * @{
  */

/** @defgroup VC_Enable_definitions
  * @{
  */
#define VC_VC25DIV_ENABLE    VC_CR0_V25DIV_EN     /*!< VC VC25 DIVIDED ENABLE */
#define VC_VC25DIV_DISABLE   (0X0U)               /*!< VC VC25 DIVIDED DISABLE */

#define VC_INT_ENABLE        VC_CR1_INT_EN        /*!< VC INTERRUPT ENABLE */
#define VC_INT_DISABLE       (0X0U)               /*!< VC INTERRUPT DISABLE */

#define VC_ENABLE            VC_CR1_VCEN          /*!< VC ENABLE */
#define VC_DISABLE           (0X0U)               /*!< VC DISABLE */
/**
  * @}
  */

/** @defgroup VC_VC25_Division_Slection
  * @{
  */
#define VC_1P4_V25DIV             (0X00U) /*!< Specifies the VC 1/4 V25D */
#define VC_2P4_V25DIV             (0x01U) /*!< Specifies the VC 2/4 V25D */
#define VC_3P4_V25DIV             (0x02U) /*!< Specifies the VC 3/4 V25D */
#define VC_4P4_V25DIV             (0x03U) /*!< Specifies the VC V25D not divided */

#define IS_VC_VC25DIV(VC_VC25DIV) (((VC_VC25DIV) == VC_1P4_V25DIV) || ((VC_VC25DIV) == VC_2P4_V25DIV) || \
                                   ((VC_VC25DIV) == VC_3P4_V25DIV) || ((VC_VC25DIV) == VC_4P4_V25DIV))
/**
  * @}
  */

/** @defgroup VC_Input_Selection
  * @{
  */
#define VC_INPUT_0                 (0X00U)
#define VC_INPUT_1                 (0X01U)
#define VC_INPUT_2                 (0X02U)
#define VC_INPUT_V25DIV            (0X03U)

#define IS_VC_INPUT(VC_INPUT)      (((VC_INPUT) == VC_INPUT_0) || ((VC_INPUT) == VC_INPUT_1) || \
                                    ((VC_INPUT) == VC_INPUT_2) || ((VC_INPUT) == VC_INPUT_V25DIV))
/**
  * @}
  */

/** @defgroup VC_Filter_Function_Selection
  * @{
  */
#define VC_FILTER_ENABLE             VC_CR1_FLTEN         /*!< VC FILTER ENABLE */
#define VC_FILTER_DISABLE            (0X0U)               /*!< VC FILTER DISABLE */

#define IS_VC_FILTER_ENABLE(FILTER)  ((FILTER == VC_FILTER_ENABLE) || (FILTER == VC_FILTER_DISABLE))
#define IS_VC_FILTER_NUM(FILTER_NUM) ((FILTER_NUM >= 0U) && (FILTER_NUM <= 0XFFFFU)) 
/**
  * @}
  */

/** @defgroup VC_Interrupt_Trigger_Selection
  * @{
  */
#define VC_HIGH_INT          VC_CR1_HIGHINTEN     /*!< VC INTERRUPT HIGHT LEVEL TRIGER ENABLE */
#define VC_RISE_INT          VC_CR1_RISEINTEN     /*!< VC INTERRUPT RISE TRIGER ENABLE */
#define VC_FALL_INT          VC_CR1_FALLINTEN     /*!< VC INTERRUPT FALL TRIGER ENABLE */
#define VC_INT_MASK          (VC_HIGH_INT | VC_RISE_INT | VC_FALL_INT)

#define IS_VC_INT(VC_INT)    (((VC_INT) == VC_HIGH_INT) || ((VC_INT) == VC_RISE_INT) || \
                              ((VC_INT) == VC_FALL_INT))
/**
  * @}
  */

/** @defgroup VC_Filter_Clock_Selection
  * @{
  */
#define VC_FILTER_CLK_NONE                 (0X00U)                             /*!< VC Filter clock select NONE */
#define VC_FILTER_CLK_PCLK                 (0X01U << VC_CR1_VC_FLTCLK_SEL_Pos) /*!< VC Filter clock select PCLK */
#define VC_FILTER_CLK_LSI                  (0X02U << VC_CR1_VC_FLTCLK_SEL_Pos) /*!< VC Filter clock select LSI */

#define IS_VC_FILTER_CLOCK(FILTER_CLOCK)   ((FILTER_CLOCK == VC_FILTER_CLK_NONE) || \
                                            (FILTER_CLOCK == VC_FILTER_CLK_PCLK) || \
                                            (FILTER_CLOCK == VC_FILTER_CLK_LSI))
/**
  * @}
  */

/** @defgroup VC_Out_Configuration
  * @{
  */
#define VC_OUTCFG_INV_PAD                  (0x01 << 18)
#define VC_OUTCFG_TM1_BKE                  (0x01 << 17)
#define VC_OUTCFG_TM1CH4_GATE              (0x01 << 16)
#define VC_OUTCFG_INV_TIM1CH4_GATE         (0x01 << 15)
#define VC_OUTCFG_TM1CH3_GATE              (0x01 << 14)
#define VC_OUTCFG_INV_TIM1CH3_GATE         (0x01 << 13)
#define VC_OUTCFG_TIM1CH2_GATE             (0x01 << 12)
#define VC_OUTCFG_INV_TIM1CH2_GATE         (0x01 << 11)
#define VC_OUTCFG_TIM1CH1_GATE             (0x01 << 10)
#define VC_OUTCFG_INV_TIM1CH1_GATE         (0x01 << 9)
#define VC_OUTCFG_PCA_EXTCLK_EN            (0x01 << 8)
#define VC_OUTCFG_PCA_CAP0_EN              (0x01 << 7)
#define VC_OUTCFG_INV_PCA                  (0x01 << 6)
#define VC_OUTCFG_LPTIM_EXTCLK_EN          (0x01 << 5)
#define VC_OUTCFG_LPTIM_GATE               (0x01 << 4)
#define VC_OUTCFG_TIM11_GATE               (0x01 << 2)
#define VC_OUTCFG_TIM10_GATE               (0x01 << 1)
#define VC_OUTCFG_INV_TIM10_TIM11          (0x01 << 0)

#define VC_OUTCFG_MSK                      (0X7FFFFU)
#define IS_VC_OUTCFG(OUTCFG)               ((OUTCFG & VC_OUTCFG_MSK) != 0X0000U)
/**
  * @}
  */

/**
  * @}
  */

/* Exported macros -----------------------------------------------------------*/
/* Exported functions --------------------------------------------------------*/

void VC_DeInit(void);
void VC_Init(VC_InitTypeDef *InitStruct);
void VC_ITConfig(uint32_t Config, FunctionalState NewState);
void VC_ITCmd(FunctionalState NewState);
void VC_OutputConfig(uint32_t Config, FunctionalState NewState);
FlagStatus VC_GetFlagStatus(void);
ITStatus VC_GetITStatus(void);
void VC_ClearITPendingBit(void);
void VC_ClearFlag(void);
FlagStatus VC_GetOutputStatus(void);
void VC_Cmd(FunctionalState NewState);

/**
  * @}
  */

/**
  * @}
  */

#ifdef __cplusplus
}
#endif

#endif /* __MG32L003_VC_H */
