/**
 * @file    mg32l003_vc.c
 * @author  megawin Application Team
 * @version V0.0.2
 * @date    16-March-2023
 * @brief   This file provides all the VC firmware functions.
 */

/* Includes ------------------------------------------------------------------*/
#include "mg32l003_vc.h"
#include "mg32l003_rcc.h"

/** @addtogroup MG32L003_StdPeriph_Driver
  * @{
  */

/** @defgroup VC
  * @brief VC driver modules
  * @{
  */

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/** @defgroup VC_Private_Functions
  * @{
  */

/**
  * @brief  Deinitializes the VC peripheral.
  * @param  None
  * @retval None
  */
void VC_DeInit(void)
{
  RCC_APBPeriphResetCmd(RCC_APBPeriph_LVDVC, ENABLE);
  RCC_APBPeriphResetCmd(RCC_APBPeriph_LVDVC, DISABLE);
}

/**
  * @brief  Initializes the VC according to the specified parameters in the
  *         VC_HandleTypeDef. 
  * @param  InitStruct: Pointer to a VC_InitTypeDef structure that contains
  *         the configuration information for the specified VC module.
  * @retval None
  */
void VC_Init(VC_InitTypeDef *InitStruct)
{
  /* Check the parameters */
  assert_param(IS_VC_VC25DIV(InitStruct->V25div));
  assert_param(IS_VC_INPUT(InitStruct->NegPinSel));
  assert_param(IS_VC_INPUT(InitStruct->PosPinSel));
  assert_param(IS_VC_FILTER_ENABLE(InitStruct->Filter_Enable));

  /* Write CR0 */
  WRITE_REG(VC->CR0, (((InitStruct->V25div) << VC_CR0_V25DIV_Pos) |
                      ((InitStruct->NegPinSel) << VC_CR0_NINSEL_Pos) |
                      (InitStruct->PosPinSel) | (VC_VC25DIV_ENABLE)));

  /* Write to CR1 about filter */
  if (InitStruct->Filter_Enable != VC_FILTER_ENABLE)
  {
    CLEAR_BIT(VC->CR1, VC_FILTER_ENABLE);
  }
  else
  {
    /* Check the parameters */
    assert_param(IS_VC_FILTER_NUM(InitStruct->Fliter_Num));
    assert_param(IS_VC_FILTER_CLOCK(InitStruct->Filter_Clk));
    /* Update CR1 with filter configuration */
    SET_BIT(VC->CR1, (((InitStruct->Fliter_Num) << VC_CR1_FLT_NUM_Pos) |
                      (InitStruct->Filter_Clk) | VC_FILTER_ENABLE));
    if ((InitStruct->Filter_Clk) == VC_FILTER_CLK_NONE)
      CLEAR_BIT(VC->CR1, VC_CR1_VC_FLTCLK_SEL);
  }
}

/**
  * @brief  Enables or disables the specified VC interrupts.
  * @param  VC_INT: VC interrupt configuration
  *         This parameter can be any combination of the following values;
  *         @arg @ref VC_HIGH_INT: VC filter ouput high level interrupt.
  *         @arg @ref VC_RISE_INT: VC filter ouput rise interrupt.
  *         @arg @ref VC_FALL_INT: VC filter ouput fall interrupt.
  * @param  NewState: The new state of the specifed VC interrupt.
  *         This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void VC_ITConfig(uint32_t VC_INT, FunctionalState NewState)
{
  /* Check the parameters */
  assert_param(IS_FUNCTIONAL_STATE(NewState));
  assert_param(IS_VC_INT(VC_INT));

  if (NewState != DISABLE)
  {
    SET_BIT(VC->CR1, VC_INT);
  }
  else
  {
    CLEAR_BIT(VC->CR1, VC_INT);
  }
}

/**
  * @brief  Enables or disables VC interrupt.
  * @param  NewState: The new state of the VC interrupt.
  *         This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void VC_ITCmd(FunctionalState NewState)
{
  /* Check the parameters */
  assert_param(IS_FUNCTIONAL_STATE(NewState));

  if (NewState != DISABLE)
  {
    SET_BIT(VC->CR1, VC_INT_ENABLE);
  }
  else
  {
    CLEAR_BIT(VC->CR1, VC_INT_ENABLE);
  }
}

/**
  * @brief  Enables or disables the specified VC outputs.
  * @param  OutputConfig: VC output configuration
  *         This parameter can be any combination of the following values:
  *         @arg @ref VC_OUTCFG_INV_PAD: VC filter output exported to PAD inverse enable.
  *         @arg @ref VC_OUTCFG_TM1_BKE: VC filter output exported to TIM1 brake control.
  *         @arg @ref VC_OUTCFG_TM1CH4_GATE: VC filter output exported to TIM1 CH4 gate control.
  *         @arg @ref VC_OUTCFG_INV_TIM1CH4_GATE: VC filter output exported to TIM CH4 inverse gate control.
  *         @arg @ref VC_OUTCFG_TM1CH3_GATE: VC filter output exported to TIM1 CH3 gate control.
  *         @arg @ref VC_OUTCFG_INV_TIM1CH3_GATE: VC filter output exported to TIM1 CH3 inverse gate control.
  *         @arg @ref VC_OUTCFG_TIM1CH2_GATE: VC filter output exported to TIM1 CH2 gate control.
  *         @arg @ref VC_OUTCFG_INV_TIM1CH2_GATE: VC filter output exported to TIM1 CH2 inverse gate control.
  *         @arg @ref VC_OUTCFG_TIM1CH1_GATE: VC filter output exported to TIM1 CH1 gate control.
  *         @arg @ref VC_OUTCFG_INV_TIM1CH1_GATE: VC filter output exported to TIM1 CH1 inverse gate control.
  *         @arg @ref VC_OUTCFG_PCA_EXTCLK_EN: VC filter output exported to PCA ECI clock.
  *         @arg @ref VC_OUTCFG_PCA_CAP0_EN: VC filter output exported to PCA capture channel 0.
  *         @arg @ref VC_OUTCFG_INV_PCA: VC filter output exported to PCA inverse.
  *         @arg @ref VC_OUTCFG_LPTIM_EXTCLK_EN: VC filter output exported to LPTIM EXT clock
  *         @arg @ref VC_OUTCFG_LPTIM_GATE: VC filter output exported to LPTIM gate control.
  *         @arg @ref VC_OUTCFG_TIM11_GATE: VC filter output exported to TIM11 gate control.
  *         @arg @ref VC_OUTCFG_TIM10_GATE: VC filter output exported to TIM10 gate control.
  *         @arg @ref VC_OUTCFG_INV_TIM10_TIM11: VC filter output exported to TIM11, TIM10 inverse gate control. 
  * @param  NewState: The new state of the specified VC outputs.
  *         This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void VC_OutputConfig(uint32_t OutputConfig, FunctionalState NewState)
{
  /* config the VC output */
  assert_param(IS_VC_OUTCFG(OutputConfig));

  if (NewState != DISABLE)
  {
    SET_BIT(VC->OUTCFG, (OutputConfig & VC_OUTCFG_MSK));
  }
  else
  {
    CLEAR_BIT(VC->OUTCFG, (OutputConfig & VC_OUTCFG_MSK));
  }
}

/**
  * @brief  Checks whether VC interrupt flag is set or reset.
  * @param  None
  * @retval The new state of VC interrupt flag (SET or RESET).
  */
FlagStatus VC_GetFlagStatus(void)
{
  FlagStatus bitstatus = RESET;

  if ((VC->SR & VC_SR_INTF))
  {
    bitstatus = SET;
  }
  else
  {
    bitstatus = RESET;
  }

  /* Return the interrupt flag status */
  return bitstatus;
}

/**
  * @brief  Checks whether VC interrupt is set or reset.
  * @param  None
  * @retval The new state of VC interrupt(SET or RESET).
  */
ITStatus VC_GetITStatus(void)
{
  ITStatus bitstatus = RESET;

  if ((VC->SR & VC_SR_INTF) && (VC->CR1 & VC_INT_ENABLE))
  {
    bitstatus = SET;
  }
  else
  {
    bitstatus = RESET;
  }

  /* Return the interrupt status */
  return bitstatus;
}

/**
  * @brief  Clears VC interrupt flag.
  * @param  None
  * @retval None
  */
void VC_ClearFlag(void)
{
  CLEAR_BIT(VC->SR, VC_SR_INTF);
}

/**
  * @brief  Clears VC interrupt.
  * @param  None
  * @retval None
  */
void VC_ClearITPendingBit(void)
{
  CLEAR_BIT(VC->SR, VC_SR_INTF);
}

/**
  * @brief  Checks VC output is set or reset.
  * @param  None
  * @retval The new state of VC_OUTPUT (SET or RESET).
  */
FlagStatus VC_GetOutputStatus(void)
{
  FlagStatus bitstatus = RESET;

  if ((VC->SR & VC_SR_VC_FLOUT) != (uint32_t)RESET)
  {
    bitstatus = SET;
  }
  else
  {
    bitstatus = RESET;
  }
  /* Return the vc output status */
  return bitstatus;
}

/**
  * @brief  Enables or disables VC peripheral(write access to VC_CR1 registers).
  * @param  NewState: The new state of the VC peripheral.
  *         This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void VC_Cmd(FunctionalState NewState)
{
  /* Check the parameters */
  assert_param(IS_FUNCTIONAL_STATE(NewState));

  if (NewState != DISABLE)
  {
    SET_BIT(VC->CR1, VC_ENABLE);
  }
  else
  {
    CLEAR_BIT(VC->CR1, VC_ENABLE);
  }
}

/**
  * @}
  */

/**
  * @}
  */

/**
  * @}
  */
