/* Includes ------------------------------------------------------------------*/
#include "drv_eeprom_24c02.h"

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
extern EEPROM_Status eeprom_status;
extern EEPROM_Stage eeprom_stage;
extern uint8_t pWData[3];
extern uint8_t pRData[2];
extern uint32_t target_size;
uint32_t transmit_size = 0;
uint32_t receive_size = 0;
uint32_t i2c_flag = 0XFF;
/* Private function prototypes -----------------------------------------------*/
I2CStatus I2C_WaitFlag(uint32_t *I2C_flag);
void eeprom_24c02_init(void);

/* Private functions ---------------------------------------------------------*/

/* This interrupt service program is for master-transmit mode */
void I2C_IRQHandler(void)
{
  I2C_ClearITPendingBit();
  I2C_WaitFlag(&i2c_flag);
  
  /* If I2C is transmitting data */
  if ((eeprom_stage == EEPROM_SEND) || (eeprom_stage == EEPROM_ADDR_BEFORE_RX))
  {
    if ((i2c_flag == I2C_FLAG_MASTER_TX_START) || (i2c_flag == I2C_FLAG_MASTER_TX_RESTART))
    {
      I2C_WriteByte(EEPROM_DEVICE_ADDRESS & (~I2C_Direction_Receiver));
      I2C_Start(DISABLE);  // No Start
    }
    else if (i2c_flag == I2C_FLAG_MASTER_TX_SLAW_ACK)
    {
      transmit_size++;
      I2C_WriteByte(pWData[transmit_size - 1]);
    }
    else if (i2c_flag == I2C_FLAG_MASTER_TX_DATA_ACK)
    { 
      if (transmit_size < target_size)
      {
        transmit_size++;
        I2C_WriteByte(pWData[transmit_size -1]);
      }
      else if (transmit_size == target_size)
      { 
        /* If you put 'transmit_size = 0' in there, the I2C_Stop will 
            set progarm into i2c_isp and transmit data again  */
        transmit_size ++;
        I2C_Stop(ENABLE);
      }
      else if (transmit_size == (target_size + 1))
      { 
        /* Data are tranmitted completely */
        transmit_size = 0;
        
        if (eeprom_stage == EEPROM_SEND)
          eeprom_status = EEPROM_SEND_RIGHT;
        else if (eeprom_stage == EEPROM_ADDR_BEFORE_RX)
          eeprom_status = EEPROM_ADDR_BEFORE_RX_RIGHT;
        else
          eeprom_status = EEPROM_GET_ERROR(eeprom_stage);
      }
      
      else
        eeprom_status = EEPROM_GET_ERROR(eeprom_stage);
    }
    else if (i2c_flag == I2C_FLAG_MASTER_TX_DATA_NOACK)
    { 
      eeprom_status = EEPROM_GET_ERROR(eeprom_stage);
    }
    else if (i2c_flag == I2C_FLAG_MASTER_TX_SLAW_NOACK)
    {
      I2C_Stop(ENABLE);
      eeprom_status = EEPROM_GET_ERROR(eeprom_stage);
    }
    
    else if (i2c_flag == I2C_FLAG_MASTER_TX_LOST_SCL)
    {
      I2C_Stop(ENABLE);
      eeprom_status = EEPROM_GET_ERROR(eeprom_stage);
    }
    else
      eeprom_status = EEPROM_GET_ERROR(eeprom_stage);
  }
  
  /* If I2C is receiving datas */
  if (eeprom_stage == EEPROM_RECEIVE)
  {
    if ((i2c_flag == I2C_FLAG_MASTER_RX_START) || (i2c_flag == I2C_FLAG_MASTER_RX_RESTART))
    { 
      I2C_WriteByte(EEPROM_DEVICE_ADDRESS | I2C_Direction_Receiver);
      I2C_Start(DISABLE);  // No Start
    }
    
    else if (i2c_flag == I2C_FLAG_MASTER_RX_SLAW_ACK)
    { 
      I2C_ACK(ENABLE);
    }
    else if (i2c_flag == I2C_FLAG_MASTER_RX_DATA_ACK)
    { 
      if (receive_size <= (target_size - 1))
      {
        I2C_ReceiveByte(&pRData[receive_size++]);
        I2C_ACK(ENABLE);
      }
      if (receive_size == (target_size - 1))
      {
        I2C_ACK(DISABLE);
      }
    }
    else if (i2c_flag == I2C_FLAG_MASTER_RX_DATA_NOACK)
    { 
      /* This judgment statement is to avoid array out of bounds,
         because I2C_STOP will set the program into isp again */
      if (receive_size == (target_size - 1))
      {
        I2C_ReceiveByte(&pRData[receive_size++]);
        eeprom_status = EEPROM_RECEIVE_RIGHT;
        I2C_Stop(ENABLE);
      }
    }
  }
}

/**
  * @brief  Initializes eeprom_24c02 and start communication.
  * @param  None
  * @retval None
  */
void eeprom_24c02_init(void) 
{
  I2C_InitTypeDef I2C_InitStruct = {0};
    
  RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOC, ENABLE);
  RCC_APBPeriphClockCmd(RCC_APBPeriph_I2C, ENABLE);
  
  /*
    PC3 (I2C_SDA)
    PC4 (I2C_SCL)
  */
  GPIO_Init(GPIOC, GPIO_Pin_3 | GPIO_Pin_4, GPIO_SPEED_LOW | GPIO_PUPD_NOPULL | GPIO_OTYPE_OD | GPIO_DRV_LOW);
  GPIO_SetAFPin(GPIOC, GPIO_Pin_3 | GPIO_Pin_4, GPIO_AF4);

  //I2C configurtion
  I2C_InitStruct.broadack = I2C_BROAD_ACK_DISABLE;
  I2C_InitStruct.master = I2C_MASTER_MODE_ENABLE;
  I2C_InitStruct.slave = I2C_SLAVE_MODE_DISABLE;
  I2C_InitStruct.speedclock = 100;  // unit khz
  I2C_Init(&I2C_InitStruct);
  
  I2C_Cmd(ENABLE);
  I2C_ClearITPendingBit();
  NVIC_SetPriority(I2C_IRQn, 2);
  NVIC_EnableIRQ(I2C_IRQn);

  eeprom_status = EEPROM_NONE;
  eeprom_stage = EEPROM_SEND;
  I2C_Start(ENABLE);
}


/**
  * @brief  Checks whether the last I2C module  status flag is get or not in timeout.
  * @param  I2C_flag: Pointer to I2C flag.
  * @retval An ErrorStatus enumeration value:
  *         - I2C_OK: Get the status flag
  *         - I2C_ERROR: Can't get the status flag
  */
I2CStatus I2C_WaitFlag(uint32_t *I2C_flag)
{
  uint32_t time_cnt = 0;
  I2CStatus ret = I2C_WAITING;
  
  /* Wait flag for 10 ms */
  do
  {
    /* Insert a delay time to get the status correctly */
    SysTick_DelayUs(100);
    /* I2C_SUCCESS: Get the i2c module status */
    ret = I2C_CheckFlag(I2C_flag);
    time_cnt++;
  } while((time_cnt < 100) && (ret == I2C_WAITING));
  
  if (ret == I2C_SUCCESS)
      return I2C_OK;
  else
      return I2C_ERROR;
}

